// (C) 2003 Ulrich Hecht <uli@emulinks.de>
// Licensed under the terms of the GNU Public License version 2
#ifndef F2A_H
#define F2A_H

int usb_connect();

void usb_disconnect();

int usb_read(char *buffer, int buffer_size);

int usb_write(char *buffer, int buffer_size);

typedef enum {
	DEVICE_USB = 0,
	DEVICE_PARPORT = 1,
        DEVICE_PPDEV = 2
} DEVICE_T;

extern DEVICE_T device;

typedef struct
{
	unsigned int command;				// command to execute, see below
	unsigned int size;					// size of data block to read/write
	unsigned int _reserved1[2];
	unsigned int magic;					// magic number, see below
	unsigned int _reserved2[3];
	unsigned int unknown;				// no idea what this is for, seems to have to be 0xa for write, 0x7 for read
	unsigned int address;				// base address for read/write
	unsigned int sizekb;				// size of data block to read/write in kB
	unsigned char _reserved3[5*4-1];	// for some reason the original software uses a 63 bytes structure for outgoing messages,
										// not 64 as it does for incoming messages, hence the "-1". It all seems to work fine with
										// 64 bytes, too, and I therefore suspect this to be a bug in the original SW.
} __attribute__ ((packed)) sendmsg;

typedef struct
{
	unsigned char data[64];
} recvmsg;

#define CMD_GETINF 5				// get info on the system status
#define CMD_WRITEDATA 6				// write data to RAM/ROM/SRAM
#define CMD_READDATA 7				// read data from RAM/ROM/SRAM
#define CMD_MULTIBOOT1 0xff			// boot up the GBA stage 1, no parameters
#define CMD_MULTIBOOT2 0			// boot up the GBA stage 2, sendmsg->size has to be set

#define MAGIC_NUMBER 0xa46e5b91		// needs to be properly set for almost all commands

#define sendf2a(x) usb_write((char*)&x,sizeof(sendmsg))
#define recvf2a(x) usb_read((char*)x,sizeof(recvmsg))

int f2a_boot(char* bootfile);
int f2a_boot();
int f2a_writemem(int numfiles, char** files, int address);
int f2a_readmem(char* file, int address, int size);

extern int verbose;

#endif
